import { resolveClusterFromEndpoint, lamports, isZeroAmount } from '@lorisleiva/js-core';
import { toWeb3JsPublicKey, fromWeb3JsPublicKey, fromBase58, toBase58 } from '@lorisleiva/js-web3js-adapters';
import { Connection } from '@solana/web3.js';

const ACCOUNT_HEADER_SIZE = 128n;
class Web3JsRpc {
  constructor(context, endpoint, rpcOptions) {
    this.context = context;
    this.connection = new Connection(endpoint, rpcOptions);
    this.cluster = resolveClusterFromEndpoint(endpoint);
  }
  getEndpoint() {
    return this.connection.rpcEndpoint;
  }
  getCluster() {
    return this.cluster;
  }
  async getAccount(address, options = {}) {
    const account = await this.connection.getAccountInfo(toWeb3JsPublicKey(address), options);
    return this.parseMaybeAccount(account, address);
  }
  async getAccounts(addresses, options = {}) {
    const accounts = await this.connection.getMultipleAccountsInfo(addresses.map(toWeb3JsPublicKey), options);
    return accounts.map((account, index) => this.parseMaybeAccount(account, addresses[index]));
  }
  async getProgramAccounts(programId, options = {}) {
    const accounts = await this.connection.getProgramAccounts(toWeb3JsPublicKey(programId), {
      ...options,
      filters: options.filters?.map(filter => this.parseDataFilter(filter))
    });
    return accounts.map(({
      pubkey,
      account
    }) => this.parseAccount(account, fromWeb3JsPublicKey(pubkey)));
  }
  async getBalance(address, options = {}) {
    const balanceInLamports = await this.connection.getBalance(toWeb3JsPublicKey(address), options);
    return lamports(balanceInLamports);
  }
  async getRent(bytes, options = {}) {
    const rentFor = bytes => this.connection.getMinimumBalanceForRentExemption(bytes, options.commitment);
    if (options.includesHeaderBytes ?? false) {
      const headerRent = await rentFor(0);
      const rentPerByte = ACCOUNT_HEADER_SIZE / BigInt(headerRent);
      return lamports(rentPerByte * BigInt(bytes));
    }
    return lamports(await rentFor(bytes));
  }
  async getLatestBlockhash(options = {}) {
    return this.connection.getLatestBlockhash(options);
  }
  async accountExists(address, options = {}) {
    return !isZeroAmount(await this.getBalance(address, options));
  }
  async airdrop(address, amount, options = {}) {
    const signature = await this.connection.requestAirdrop(toWeb3JsPublicKey(address), Number(amount.basisPoints));
    if (options.strategy) {
      this.confirmTransaction(fromBase58(signature), options);
      return;
    }
    this.confirmTransaction(fromBase58(signature), {
      ...options,
      strategy: {
        type: 'blockhash',
        ...(await this.getLatestBlockhash())
      }
    });
  }
  async call(method, params, options = {}) {
    const client = this.connection._rpcClient;
    return new Promise((resolve, reject) => {
      const callback = (error, result) => error ? reject(error) : resolve(result);
      client.request(method, params, options.id ?? null, callback);
    });
  }
  async sendTransaction(serializedTransaction, options = {}) {
    try {
      const signature = await this.connection.sendRawTransaction(serializedTransaction, options);
      return fromBase58(signature);
    } catch (error) {
      let resolvedError = null;
      if (error instanceof Error && 'logs' in error) {
        resolvedError = this.context.programs.resolveError(error);
      }
      throw resolvedError || error;
    }
  }
  async confirmTransaction(signature, options) {
    return this.connection.confirmTransaction(this.parseConfirmStrategy(signature, options), options.commitment);
  }
  parseAccount(account, address) {
    return {
      ...account,
      owner: fromWeb3JsPublicKey(account.owner),
      address,
      lamports: lamports(account.lamports)
    };
  }
  parseMaybeAccount(account, address) {
    return account ? {
      ...this.parseAccount(account, address),
      exists: true
    } : {
      exists: false,
      address
    };
  }
  parseDataFilter(filter) {
    if (!('memcmp' in filter)) return filter;
    const {
      bytes,
      ...rest
    } = filter.memcmp;
    return {
      memcmp: {
        ...rest,
        bytes: toBase58(bytes)
      }
    };
  }
  parseConfirmStrategy(signature, options) {
    if (options.strategy.type === 'blockhash') {
      return {
        ...options.strategy,
        signature: toBase58(signature)
      };
    }
    return {
      ...options.strategy,
      signature: toBase58(signature),
      nonceAccountPubkey: toWeb3JsPublicKey(options.strategy.nonceAccountPubkey)
    };
  }
}

export { ACCOUNT_HEADER_SIZE, Web3JsRpc };
//# sourceMappingURL=Web3JsRpc.mjs.map
